# -*- coding: utf-8 -*-
# Copyright 2017 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import os
import unittest

from acsoo.config import AcsooConfig


DATA_DIR = os.path.join(os.path.dirname(__file__), 'data')


class TestConfig(unittest.TestCase):

    def setUp(self):
        self.config = AcsooConfig(os.path.join(DATA_DIR, 'test1.cfg'))

    def test1(self):
        assert self.config.get('test', 'absent') is None
        assert self.config.get('test', 'absent', 'default') == 'default'
        assert self.config.get('test', 'bool') == 'yes'
        assert self.config.get('test', 'multi') == '\nABC\nDEF'
        assert self.config.get('test', 'multi', flatten=True) == 'ABCDEF'
        assert self.config.getboolean('test', 'bool') is True
        assert self.config.getboolean('test', 'absent') is None
        assert self.config.getboolean('test', 'absent', False) is False
        assert self.config.getlist('test', 'multi') == ['ABC', 'DEF']
        assert self.config.getlist('test', 'absent') == []
        assert self.config.getlist('test', 'absent', ['A', 'B']) == ['A', 'B']
        assert self.config.get('absent', 'absent') is None
        assert self.config.getboolean('absent', 'absent') is None
        assert self.config.getlist('absent', 'absent') == []

    def test2(self):
        assert self.config.trigram == 'xyz'
        assert self.config.version == '1.1.0'
        assert self.config.series == '10.0'

    def test_default_map(self):
        default_map = self.config.get_default_map()
        assert default_map['checklog'] == {
            'ignore': ['WARNING', 'ERROR:.*registry'],
            'echo': None,
        }



# -*- coding: utf-8 -*-
# Copyright 2017 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import logging
import sys
from configparser import ConfigParser

import click
import pylint.lint

from .config import AcsooConfig
from .main import main
from .manifest import get_installable_addons
from .tools import cmd_string, log_cmd, cfg_path


def _get_failures(linter_stats, expected):
    fails = []
    no_fails = []
    for msg, count in linter_stats['by_msg'].items():
        if not count:
            continue
        if msg in expected:
            expected_count = expected[msg]
            if expected_count is None or count == expected_count:
                no_fails.append((msg, count, expected_count))
            else:
                fails.append((msg, count, expected_count))
        else:
            fails.append((msg, count, None))
    return (sorted(fails, key=lambda i: i[0]),
            sorted(no_fails, key=lambda i: i[0]))


def _failures_to_str(fails, no_fails):
    def _r(l):
        for msg, count, expected_count in l:
            res.append('  {}: {}'.format(msg, count))
            if expected_count is not None:
                res.append(' (expected {})'.format(expected_count))
            res.append('\n')

    res = []
    if no_fails:
        res.append(click.style(
            'messages that did not cause failure:\n',
            bold=True))
        _r(no_fails)
    if fails:
        res.append(click.style(
            'messages that caused failure:\n',
            bold=True))
        _r(fails)
    return ''.join(res)


def _parse_msg_string(s):
    res = {}
    s = s or ''
    for msg in s.split(','):
        msg = msg.strip()
        if not msg:
            continue
        if ':' in msg:
            msg, count = msg.split(':', 2)
            msg = msg.strip()
            count = int(count)
            res[msg] = count
        else:
            res[msg] = None
    return res


def _consolidate_expected(rcfile, expected):
    config = ConfigParser()
    config.read([rcfile])
    res = {}
    if config.has_option('ACSOO', 'expected'):
        res.update(_parse_msg_string(config.get('ACSOO', 'expected')))
    res.update(expected)
    return res


def do_pylintcmd(load_plugins, rcfile, module, expected, pylint_options):
    # import pdb; pdb.set_trace()
    if not module:
        module = []
        addons = get_installable_addons()
        module.extend(
            addon_dir
            for addon_name, (addon_dir, manifest) in addons.items()
        )
        if not module:
            raise click.UsageError("Please provide module or package "
                                   "to lint (--module).")
    cmd = [
        '--load-plugins', load_plugins,
        '--rcfile', rcfile,
    ] + list(pylint_options) + list(module)
    log_cmd(['pylint'] + cmd, level=logging.INFO)
    lint_res = pylint.lint.Run(cmd[:], exit=False)
    sys.stdout.flush()
    sys.stderr.flush()
    expected = _consolidate_expected(rcfile, expected)
    fails, no_fails = _get_failures(lint_res.linter.stats, expected)
    if fails or no_fails:
        msg = cmd_string(['pylint'] + cmd)
        msg += '\n'
        msg += _failures_to_str(fails, no_fails)
        click.echo('\n')
        click.echo(msg)
    if fails:
        raise click.ClickException("pylint errors detected.")


@click.command()
@click.option('--module', '-m', metavar='module_or_package', multiple=True,
              help="Module or package to lint (default: autodetected "
                   "odoo/addons or odoo_addons or in current directory).")
@click.option('--load-plugins', metavar='PLUGINS', default='pylint_odoo',
              help="Pylint plugins to use (default: pylint_odoo).")
@click.option('--rcfile', type=click.Path(dir_okay=False, exists=True),
              default=cfg_path('pylint.cfg'),
              help="Pylint configuration file. Default is provided by acsoo.")
@click.option('--expected', '-e', 'expected', metavar='MSG-IDS',
              help="Do not fail on these messages.")
@click.argument('pylint-options', nargs=-1)
@click.pass_context
def pylintcmd(ctx, load_plugins, rcfile, module, expected, pylint_options):
    """Run pylint with reasonable defaults.

    You may pass additional
    options to pylint using '--' to separate them from
    acsoo options.

    Default options are read from the
    [pylint] section of the acsoo configuration file.
    The pylint-options key in that section can be used
    to provide default additional pylint options (one
    per line). Example configuration file:

    \b
    [pylint]
    expected=fixme:5
    pylint-options=
      --disable=all
      --enable=fixme

    The above configuration file is equivalent to:

    acsoo pylint --expected=fixme:5 -- --disable=all --enable=fixme
    """
    expected = _parse_msg_string(expected)
    default_pylint_options = (ctx.default_map or {}).\
        get('default_pylint_options', [])
    pylint_options = default_pylint_options + list(pylint_options)
    do_pylintcmd(load_plugins, rcfile, module, expected, pylint_options)


main.add_command(pylintcmd, name='pylint')


def _read_defaults(config):
    section = 'pylint'
    defaults = dict(
        module=config.getlist(
            section, 'module'),
        load_plugins=config.get(
            section, 'load-plugins', default='pylint_odoo'),
        rcfile=config.get(
            section, 'rcfile', default=cfg_path('pylint.cfg')),
        expected=config.get(
            section, 'expected', flatten=True),
        default_pylint_options=config.getlist(
            section, 'pylint-options'),
    )
    return dict(pylint=defaults)


AcsooConfig.add_default_map_reader(_read_defaults)



# -*- coding: utf-8 -*-
# Copyright 2016 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import click

from .main import main
from .tools import check_call, call, check_output
from .tag_requirements import do_tag_requirements


def do_tag(config, force, src, requirement, yes):
    tag = config.version
    if not yes:
        click.confirm('Tag project with {}?'.format(tag), abort=True)
    if force:
        force_cmd = ['-f']
    else:
        force_cmd = []
    if call(['git', 'diff', '--exit-code']) != 0:
        raise click.ClickException("Please commit first.")
    if call(['git', 'diff', '--exit-code', '--cached']) != 0:
        raise click.ClickException("Please commit first.")
    out = check_output(['git', 'ls-files', '--other', '--exclude-standard',
                        '--directory'])
    if out:
        click.echo(out)
        raise click.ClickException("Please commit first.")
    do_tag_requirements(config, force, src, requirement, yes=True)
    check_call(['git', 'tag'] + force_cmd + [tag])
    check_call(['git', 'push', '-q'] + force_cmd + ['origin', 'tag', tag])


@click.command(help='Tag the current project after ensuring '
                    'everything has been commited to git.')
@click.option('--src', default='src', envvar='PIP_SRC',
              type=click.Path(file_okay=False),
              help='Directory where editable requirements are checked out')
@click.option('-r', '--requirement', default='requirements.txt',
              type=click.Path(dir_okay=False, exists=True),
              help='Requirements to build (default=requirements.txt)')
@click.option('-f', '--force', is_flag=True,
              help='Replace an existing tag (instead of failing)')
@click.option('-y', '--yes', is_flag=True, default=False)
@click.pass_context
def tag(ctx, force, src, requirement, yes):
    """ Tag the current project and its VCS requirements.

    This command verifies everything has been committed, then
    performs git tag, git push and acsoo tag_requirements.
    """
    do_tag(ctx.obj['config'], force, src, requirement, yes)


main.add_command(tag)



# -*- coding: utf-8 -*-
# Copyright 2017 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import os
from os.path import join as opj
import unittest

from click.testing import CliRunner

from acsoo.main import main
from acsoo.tools import working_directory


DATA_DIR = opj(os.path.dirname(__file__), 'data')


class TestAddons(unittest.TestCase):

    def test_list(self):
        runner = CliRunner()
        with working_directory(DATA_DIR):
            res = runner.invoke(main, [
                'addons',
                'list',
            ])
            assert res.exit_code == 0
            expected = 'addon1,addon2\n'
            assert expected == res.output

    def test_exclude(self):
        runner = CliRunner()
        with working_directory(DATA_DIR):
            res = runner.invoke(main, [
                'addons',
                '--exclude', 'addon1',
                'list',
            ])
            assert res.exit_code == 0
            expected = 'addon2\n'
            assert expected == res.output

    def test_include(self):
        runner = CliRunner()
        with working_directory(DATA_DIR):
            res = runner.invoke(main, [
                'addons',
                '--include', 'addon1',
                'list',
            ])
            assert res.exit_code == 0
            expected = 'addon1\n'
            assert expected == res.output

    def test_list_depends(self):
        runner = CliRunner()
        with working_directory(DATA_DIR):
            res = runner.invoke(main, [
                'addons',
                'list-depends',
            ])
            assert res.exit_code == 0
            expected = 'base\n'
            assert expected == res.output

    def test_list_depends_exclude(self):
        runner = CliRunner()
        with working_directory(DATA_DIR):
            res = runner.invoke(main, [
                'addons',
                'list-depends',
                '--exclude=base',
            ])
            assert res.exit_code == 0
            expected = '\n'
            assert expected == res.output

    def test_separator(self):
        runner = CliRunner()
        with working_directory(DATA_DIR):
            res = runner.invoke(main, [
                'addons',
                '--separator', ';',
                'list',
            ])
            assert res.exit_code == 0
            expected = 'addon1;addon2\n'
            assert expected == res.output

    def test_addons_in_cur_dir(self):
        runner = CliRunner()
        with working_directory(opj(DATA_DIR, 'odoo', 'addons')):
            res = runner.invoke(main, [
                'addons',
                '--separator', ';',
                'list',
            ])
            assert res.exit_code == 0
            expected = 'addon1;addon2\n'
            assert expected == res.output



# -*- coding: utf-8 -*-
# Copyright 2016 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import click

from .main import main
from .wheel import do_wheel
from .tag import do_tag


def do_release(config, force, src, requirement, wheel_dir, yes,
               no_cache_dir, no_index):
    if not yes:
        click.confirm('Tag and release version {}?'.format(config.version),
                      abort=True)
        yes = True
    do_tag(config, force, src, requirement, yes)
    do_wheel(src, requirement, wheel_dir, no_cache_dir, no_index)


@click.command(help='Perform acsoo tag, and acsoo wheel')
@click.option('-f', '--force', is_flag=True,
              help='Replace an existing tag (instead of failing)')
@click.option('--src', default='src', envvar='PIP_SRC',
              type=click.Path(file_okay=False),
              help='Directory where editable requirements are checked out')
@click.option('-r', '--requirement', default='requirements.txt',
              type=click.Path(dir_okay=False, exists=True),
              help='Requirements to build (default=requirements.txt)')
@click.option('-w', '--wheel-dir', default='release',
              type=click.Path(file_okay=False),
              help='Path where the wheels will be created (default=release')
@click.option('-y', '--yes', is_flag=True, default=False)
@click.option('--no-cache-dir', is_flag=True,
              help='Disable the pip cache')
@click.option('--no-index', is_flag=True,
              help='Ignore package index '
                   '(only looking at --find-links URLs instead)')
@click.pass_context
def release(ctx, force, src, requirement, wheel_dir, yes,
            no_cache_dir, no_index):
    do_release(ctx.obj['config'], force, src, requirement, wheel_dir, yes,
               no_cache_dir, no_index)


main.add_command(release)



# -*- coding: utf-8 -*-
# Copyright 2016 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import logging
import re
import os

import click

from .main import main
from .tools import call, check_call, check_output, working_directory

_logger = logging.getLogger(__name__)


RE = re.compile(r"(?P<editable>-e )?(?P<vcs>(git|svn|hg))\+"
                r"(?P<url>(ssh|http|https)://.*?/.*?)@(?P<sha>[^?#&]+)"
                r".*?[#?&]egg=(?P<egg>[^?#&]+)")
NOTAG_RE = re.compile(r"([a-zA-Z0-9-_\.]+==)|"
                      r"(-f )|(--find-links )|"
                      r"(--extra-index-url )")
GIT_URL_RE = re.compile(r"(?P<scheme>ssh|http|https)://"
                        r"(?P<user>git@)?(?P<host>.*?)/"
                        r"(?P<org>.*?)/(?P<rest>.*)")
PUSHABLE = [
    ('github.com', 'acsone'),
]


def _make_push_url(url):
    mo = GIT_URL_RE.match(url)
    if not mo:
        return False
    groups = mo.groupdict()
    if groups['scheme'] == 'ssh':
        return url
    if (groups['host'].lower(), groups['org'].lower()) not in PUSHABLE:
        return False
    return 'ssh://git@{host}/{org}/{rest}'.format(**groups)


def _get_last_sha(filename):
    # find the short sha of the last commit to filename
    return check_output([
        'git', 'log', '-n', '1', '--format=%h', filename
    ]).strip()


def _has_tag(series, trigram, egg, sha, repodir='.'):
    tag_re = re.compile(series + "[-_]" + trigram +
                        "[-_][a-fA-F0-9]+[-_]" + egg)
    tags = check_output(['git', 'tag', '--points-at', sha], cwd=repodir)
    for tag in tags.split():
        tag = tag.strip()
        if tag_re.match(tag):
            return tag
    return False


def _is_committed(requirement):
    out = check_output(['git', 'ls-files', requirement])
    if not out:
        return False
    r = call(['git', 'diff', '--exit-code', requirement])
    return r == 0


def do_tag_requirements(config, force, src, requirement, yes):
    if not _is_committed(requirement):
        raise click.ClickException("Please commit %s first." % (requirement, ))
    requirement_sha = _get_last_sha(requirement)
    base_tag = '{}-{}-{}'.format(
        config.series, config.trigram, requirement_sha)
    if not yes:
        click.confirm('Tag dependencies with {}?'.format(base_tag), abort=True)
    if force:
        force_cmd = ['-f']
    else:
        force_cmd = []
    for req in open(requirement):
        req = req.strip()
        mo = RE.match(req)
        if not mo:
            if not NOTAG_RE.match(req):
                _logger.warning("Cannot not tag %s (unrecognized req)", req)
            continue
        editable = bool(mo.group('editable'))
        vcs = mo.group('vcs')
        url = mo.group('url')
        egg = mo.group('egg')
        sha = mo.group('sha')
        if vcs != 'git':
            _logger.warning("Cannot tag %s (unsupported vcs)", req)
            continue
        if not editable:
            _logger.warning("Cannot tag %s (non editable)", req)
            continue
        push_url = _make_push_url(url)
        if not push_url:
            _logger.warning("Cannot tag %s (not pushable)", req)
            continue
        repodir = os.path.join(src, egg.replace('_', '-'))
        if not os.path.isdir(os.path.join(repodir, '.git')):
            os.makedirs(repodir)
            check_call(['git', 'init'], cwd=repodir)
        with working_directory(repodir):
            ex_tag = _has_tag(config.series, config.trigram, egg, sha)
            if ex_tag:
                # this assumes that if we find the tag locally
                # it is also present on the remote, in rare situations
                # this may not be the case, but this is so much more
                # performant that it's probably worth taking this shortcut.
                click.echo('tag {ex_tag} already exists on {url}@{sha}'.
                           format(**locals()))
                continue
            check_call([
                'git', 'fetch', '-q', '-f', '--tags', url,
                '+refs/heads/*:refs/remotes/acsoo/*'])
            ex_tag = _has_tag(config.series, config.trigram, egg, sha)
            if ex_tag:
                click.echo('tag {ex_tag} already exists on {url}@{sha}'.
                           format(**locals()))
                continue
            eggtag = base_tag + '-' + egg
            click.echo('placing tag {eggtag} on {push_url}@{sha}'.
                       format(**locals()))
            check_call(['git', 'tag'] + force_cmd + [eggtag, sha])
            try:
                check_call(['git', 'push'] + force_cmd + [push_url, eggtag])
            except:  # noqa
                # if push failed, delete local tag
                call(['git', 'tag', '-d', eggtag])
                raise


@click.command()
@click.option('-f', '--force', is_flag=True,
              help='Replace an existing tag (instead of failing)')
@click.option('--src', default='src', envvar='PIP_SRC',
              type=click.Path(file_okay=False),
              help='Directory where editable requirements are checked out')
@click.option('-r', '--requirement', default='requirements.txt',
              type=click.Path(dir_okay=False, exists=True),
              help='Requirements file to use (default=requirements.txt)')
@click.option('-y', '--yes', is_flag=True, default=False)
@click.pass_context
def tag_requirements(ctx, force, src, requirement, yes):
    """ Tag all VCS requirements found in requirements.txt.

    This is important to avoid that commits referenced in requirements.txt
    are lost in case of VCS garbage collection.

    Only git is supported for now.
    """
    do_tag_requirements(
        ctx.obj['config'], force, src, requirement, yes)


main.add_command(tag_requirements)



# -*- coding: utf-8 -*-
# Copyright 2017 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import click

from .main import main
from .manifest import get_installable_addons


def _split_set(csv):
    return set(s.strip() for s in csv.split(',') if s.strip())


@click.group(help="Do things with addons lists. Options of this command "
                  "select addons on which the subcommands will act.")
@click.option('--addons-dir', 'addons_dirs', multiple=True,
              type=click.Path(file_okay=False, exists=True),
              help="Directory containing addons. Defaults to odoo/addons or "
                   "odoo_addons if present. This option can be repeated.")
@click.option('--include', default='',
              help="Comma separated list of addons to include (default: all "
                   "installable addons found in --addons-dir').")
@click.option('--exclude', default='',
              help="Comma separated list of addons to exclude.")
@click.option('--separator', '-s', default=',',
              help="Separator (default: comma)")
@click.pass_context
def addons(ctx, addons_dirs, include, exclude, separator):
    include = _split_set(include)
    exclude = _split_set(exclude)
    addons = {}
    installable_addons = get_installable_addons(addons_dirs)
    for addon_name, (addon_dir, manifest) in installable_addons.items():
        if (not include or addon_name in include) and \
                addon_name not in exclude:
            addons[addon_name] = (addon_dir, manifest)
    ctx.obj.update(dict(
        addons=addons,
        separator=separator,
    ))


main.add_command(addons)


@click.command(help="Print a comma separated list of selected addons.")
@click.pass_context
def addons_list(ctx):
    addons = ctx.obj['addons']
    addon_names = sorted(addons.keys())
    click.echo(ctx.obj['separator'].join(addon_names))


addons.add_command(addons_list, 'list')


@click.command(help="Print a comma separated list of the direct dependencies "
                    "of installable addons found in --addons-dir.")
@click.option('--exclude', default='',
              help="Comma separated list of addons to exclude from "
                   "the dependencies.")
@click.pass_context
def addons_list_depends(ctx, exclude):
    exclude = _split_set(exclude)
    addons = ctx.obj['addons']
    depends = set()
    for addon_dir, manifest in addons.values():
        depends.update(manifest.get('depends', []))
    depends -= set(addons.keys())
    depends -= exclude
    addon_names = sorted(depends)
    click.echo(ctx.obj['separator'].join(addon_names))


addons.add_command(addons_list_depends, 'list-depends')



# -*- coding: utf-8 -*-
# Copyright 2017 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import os
import unittest

from click.testing import CliRunner

from acsoo.checklog import checklog
from acsoo.main import main


DATA_DIR = os.path.join(os.path.dirname(__file__), 'data')


class TestChecklog(unittest.TestCase):

    def test1(self):
        runner = CliRunner()
        res = runner.invoke(checklog, [
            os.path.join(DATA_DIR, 'test1.log'),
        ])
        assert res.exit_code != 0
        expected = "errors that caused failure (2):"
        assert expected in res.output

    def test2(self):
        runner = CliRunner()
        res = runner.invoke(checklog, [
            '--ignore', ' ERROR ',
            os.path.join(DATA_DIR, 'test1.log'),
        ])
        assert res.exit_code != 0
        expected = "errors that caused failure (1):"
        assert expected in res.output
        expected = "errors that did not cause failure (1):"
        assert expected in res.output

    def test3(self):
        runner = CliRunner()
        res = runner.invoke(checklog, [
            '-i', ' ERROR ',
            '-i', ' CRITICAL ',
            os.path.join(DATA_DIR, 'test1.log'),
        ])
        assert res.exit_code == 0
        expected = "errors that did not cause failure (2):"
        assert expected in res.output

    def test4(self):
        runner = CliRunner()
        res = runner.invoke(main, [
            '-c', os.path.join(DATA_DIR, 'test_checklog.cfg'),
            'checklog',
            os.path.join(DATA_DIR, 'test1.log'),
        ])
        assert res.exit_code != 0
        expected = "errors that caused failure (1):"
        assert expected in res.output
        expected = "errors that did not cause failure (1):"
        assert expected in res.output

    def test_empty(self):
        runner = CliRunner()
        res = runner.invoke(checklog, [
            os.path.join(DATA_DIR, 'empty.log'),
        ])
        assert res.exit_code != 0
        expected = "No Odoo log record found in input."
        assert expected in res.output
        res = runner.invoke(checklog, [
            '--no-err-if-empty',
            os.path.join(DATA_DIR, 'empty.log'),
        ])
        assert res.exit_code == 0



# -*- coding: utf-8 -*-
# Copyright © 2015 ACSONE SA/NV
# License LGPLv3 (http://www.gnu.org/licenses/lgpl-3.0-standalone.html)

import ast
import os

MANIFEST_NAMES = ('__openerp__.py', '__manifest__.py', '__terp__.py')


class NoManifestFound(Exception):
    pass


def get_manifest_path(addon_dir):
    for manifest_name in MANIFEST_NAMES:
        manifest_path = os.path.join(addon_dir, manifest_name)
        if os.path.isfile(manifest_path):
            return manifest_path


def parse_manifest(s):
    return ast.literal_eval(s)


def get_default_addons_dirs():
    addons_dirs = []
    candidate_addons_dirs = (
        os.path.join('odoo', 'addons'),
        'odoo_addons',
        '.',
    )
    for addons_dir in candidate_addons_dirs:
        if os.path.isdir(addons_dir):
            addons_dirs.append(addons_dir)
    return addons_dirs


def get_installable_addons(addons_dirs=None):
    """
    This method builds a dictionary of all installable addons in the specified
    addons directory or in the default addons directories.
    :param addons_dirs: path to the addons directories to fetch into
    :return: Dictionary like: {'addon_name': (addons_directory, manifest)}
    where manifest is a dictionary.
    """
    if not addons_dirs:
        addons_dirs = get_default_addons_dirs()
    res = {}
    for addons_dir in addons_dirs:
        for addon_name in os.listdir(addons_dir):
            addon_dir = os.path.join(addons_dir, addon_name)
            manifest_path = get_manifest_path(addon_dir)
            if not manifest_path:
                continue
            with open(manifest_path) as f:
                manifest = parse_manifest(f.read())
            if not manifest.get('installable', True):
                continue
            res[addon_name] = (addon_dir, manifest)
    return res



# -*- coding: utf-8 -*-
# Copyright 2017 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import logging

import click

from .config import AcsooConfig
from .main import main
from .tools import check_call, cfg_path


def do_flake8(config, flake8_options):
    # flake8 __init__.py
    cmd = [
        'flake8',
        '--config', cfg_path('flake8__init__.cfg'),
    ]
    check_call(cmd, log_level=logging.INFO)
    # flake8
    cmd = [
        'flake8',
        '--config', config,
    ] + list(flake8_options)
    check_call(cmd, log_level=logging.INFO)


@click.command()
@click.option('--config', type=click.Path(dir_okay=False, exists=True),
              default=cfg_path('flake8.cfg'),
              help="Flake8 config file. Default is provided by acsoo.")
@click.argument('flake8-options', nargs=-1)
@click.pass_context
def flake8(ctx, config, flake8_options):
    """Run flake8 with reasonable defaults.

    You may pass additional
    options to flake8 using '--' to separate them from
    acsoo options.

    Default options are read from the
    [flake8] section of the acsoo configuration file.
    The flake8-options key in that section can be used
    to provide default additional flake8 options (one
    per line). Example configuration file:

    \b
    [flake8]
    pylint-options=
      --append-config=flake8.cfg

    The above configuration file is equivalent to:

    acsoo flake8 -- --append-config=flake8.cfg
    """
    default_flake8_options = (ctx.default_map or {}).\
        get('default_flake8_options', [])
    flake8_options = default_flake8_options + list(flake8_options)
    do_flake8(config, flake8_options)


main.add_command(flake8)


def _read_defaults(config):
    section = 'flake8'
    defaults = dict(
        config=config.get(
            section, 'config', default=cfg_path('flake8.cfg')),
        default_flake8_options=config.getlist(
            section, 'flake8-options'),
    )
    return dict(flake8=defaults)


AcsooConfig.add_default_map_reader(_read_defaults)



# -*- coding: utf-8 -*-
# Copyright 2017 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import logging
import re
import sys

import click

from .config import AcsooConfig
from .main import main


_logger = logging.getLogger(__name__)


# from tartley/colorama
ANSI_CSI_RE = re.compile('\001?\033\\[((?:\\d|;)*)([a-zA-Z])\002?')

# from OCA/maintainer-quality-tools
LOG_START_RE = re.compile(
    r'^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2},\d{3} \d+ (?P<loglevel>\w+) '
    r'(?P<db>\S+) (?P<logger>\S+): (?P<message>.*)$')

NON_ERROR_LEVELS = ('INFO', 'DEBUG')


def _render_errors(error_records, ignored_error_records):
    msg = []
    if ignored_error_records:
        msg.append(click.style(
            '\nerrors that did not cause failure ({}):\n'.
            format(len(ignored_error_records)),
            bold=True))
        msg.extend(ignored_error_records)
    if error_records:
        msg.append(click.style(
            '\nerrors that caused failure ({}):\n'.
            format(len(error_records)),
            bold=True))
        msg.extend(error_records)
    return ''.join(msg)


def do_checklog(filename, ignore, echo, err_if_empty=True):
    ignore = [i for i in ignore if not i.startswith('#')]
    _logger.debug("ignored regular expressions:\n%s", '\n'.join(ignore))
    ignore_regexes = [re.compile(i, re.MULTILINE) for i in ignore]

    if echo is None and filename == '-':
        echo = True

    with click.open_file(filename) as logfile:
        cur_rec_mo = None
        cur_rec = []
        error_records = []
        ignored_error_records = []

        def _process_cur_rec():
            # record start, process current record
            if cur_rec_mo and \
                    cur_rec_mo.group('loglevel') not in NON_ERROR_LEVELS:
                record = ''.join(cur_rec)
                for ignore_regex in ignore_regexes:
                    if ignore_regex.search(record):
                        ignored_error_records.append(record)
                        break
                else:
                    error_records.append(record)

        reccount = 0
        for line in logfile:
            if echo:
                click.echo(line, nl=False, color=True)
                sys.stdout.flush()
            line = ANSI_CSI_RE.sub('', line)  # strip ANSI colors
            mo = LOG_START_RE.match(line)
            if mo:
                reccount += 1
                _process_cur_rec()
                cur_rec_mo = mo
                cur_rec = [line]
            else:
                cur_rec.append(line)
        _process_cur_rec()  # last record

        if not reccount and err_if_empty:
            raise click.ClickException("No Odoo log record found in input.")

        if error_records or ignored_error_records:
            msg = _render_errors(error_records, ignored_error_records)
            click.echo(msg)
        if error_records:
            raise click.ClickException("Errors detected in log.")


@click.command(help="Check an odoo log file for errors. When no filename "
                    "or - is provided, read from stdin. Default options "
                    "are read from the [checklog] section of the acsoo "
                    "configuration file.")
@click.option('--ignore', '-i', metavar='REGEX', multiple=True,
              help="Regular expression of log records to ignore.")
@click.option('--echo/--no-echo', default=None,
              help="Echo the input file (default when reading from stdin).")
@click.option('--err-if-empty/--no-err-if-empty', default=True,
              help="Exit with an error code if no log record is found "
                   "(default).")
@click.argument('filename', type=click.Path(dir_okay=False), default='-')
def checklog(filename, ignore, echo, err_if_empty):
    do_checklog(filename, ignore, echo, err_if_empty)


main.add_command(checklog)


def _read_defaults(config):
    section = 'checklog'
    defaults = dict(
        ignore=config.getlist(section, 'ignore', []),
        echo=config.getboolean(section, 'echo', None),
    )
    return dict(checklog=defaults)


AcsooConfig.add_default_map_reader(_read_defaults)



# -*- coding: utf-8 -*-
# Copyright 2016 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

from setuptools import setup, find_packages


setup(
    name='acsoo',
    description='Acsone Odoo Dev Tools',
    long_description='\n'.join((
        open('README.rst').read(),
        open('CHANGES.rst').read(),
    )),
    use_scm_version=True,
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'bobtemplates.odoo',
        'click',
        'colorama',
        'configparser',
        'flake8',
        'lxml',  # pylint-odoo dep not installed automatically?
        'pip>=9.0.1',
        'pylint-odoo',
        'setuptools>=20,<31',
        'wheel>=0.29',
        'bumpversion',
    ],
    license='GPLv3+',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: '
        'GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2.7',
        'Framework :: Odoo',
    ],
    setup_requires=[
        'setuptools-git',
        'setuptools-scm',
    ],
    entry_points='''
        [console_scripts]
        acsoo=acsoo.main:main
    ''',
)
